from chargehound.resources import Disputes

api_key = None
base_path = '/v1/'
host = 'api.chargehound.com'
port = None
protocol = 'https://'
timeout = 60
version = None

__all__ = [
  api_key,
  Disputes,
  host,
  port,
  protocol,
  timeout,
  version
]
